#ifdef __cplusplus
extern "C" {
#endif  

#include <includes.h>
#include <ssh.h>
#include <gmp.h>
#include <xmalloc.h>
#include <rsa.h>
#include <blowfish.h>
#include <cipher.h>
#include <stdio.h>
#include <sys/timeb.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef __cplusplus
}
#endif  

#include <winscard.h>
#include <kcryptfile.h> 

extern UCHAR ACCESS_CHV0_VAL[8];                                      
extern DWORD ACCESS_CHV0_SIZE;
extern UCHAR ACCESS_AUT0_KEY[8];  
extern DWORD ACCESS_AUT0_SIZE; 

void PMSCGenerateRandomNonce( PUCHAR pucBuffer, DWORD dwLength );


void PMSCGenerateRandomNonce( PUCHAR pucBuffer, DWORD dwLength ) {

  UCHAR ucRByte;
  int i;
  int iRandom;

  /* Start with a decent seed and then feed the random
     as the seed - use the Epoch and pid.
  */

  iRandom  = time(0) + getpid();

  for ( i=0; i < dwLength; i++ ) {
    srand( iRandom + i );
    ucRByte = (1 + (int)(255.0*rand()/(RAND_MAX+1.0)));
    pucBuffer[i] = ucRByte;
  }

  return;
}

void CSCryptFile( char *srcfilename, char *destfilename, PUCHAR pucKey, 
		  PUCHAR pucCipherKey, DWORD dwSize ) {

  int fda, fdb, readlen, writelen, i;
  char cfile[200];
  unsigned char src[200], dest[200];
  unsigned char ukey[200];
  CipherContext sCipher;
  struct stat fbuffer;
  LONG filesize;
  LONG lByteCnt;

  lByteCnt = 0;

  fda = open( srcfilename, O_RDONLY );

  if ( fda <= 0 ) {
    printf("Error opening file: %s\n", srcfilename);
    exit(1);
  }

  fdb = open( destfilename,  O_CREAT | O_WRONLY, S_IRWXU | S_IROTH );

  if ( fdb <= 0 ) {
    printf("Error opening file: %s\n", destfilename);
    exit(1);
  }
  
  cipher_set_key( &sCipher, SSH_CIPHER_IDEA, pucKey, 16, 1 );

  fstat( fda, &fbuffer );
  filesize = fbuffer.st_size;

  /* Cipher type */
  dest[0] = SSH_CIPHER_IDEA;

  /* Data size */
  dest[1] = filesize / 0x01000000;
  filesize = filesize - dest[1]*0x01000000;

  dest[2] = filesize / 0x00010000;
  filesize = filesize - dest[2]*0x00010000;

  dest[3] = filesize / 0x00000100;
  filesize = filesize - dest[3]*0x00000100;

  dest[4] = filesize;

  dest[5] = 0x80;  /* Size of encrypted key */
  dest[6] = 0x16;  /* Actual size of key    */

  dest[7] = 0x00;  /* RFU - hashing possibly */
  dest[8] = 0x00;  /* RFU - hashing possibly */

  /* Copy the 1024 bit RSA key over */
  memcpy( &dest[9], pucCipherKey, dest[5] );


  /* 5 byte header with 1024 bit key */
  writelen = write( fdb, dest, 0x80 + 4 + 2 + 2 + 1 );

  do {
    
    readlen = read( fda, src, 16 ); 
    
    if ( readlen > 0 ) {
      /* Read, encrypt, and repeat */
      cipher_encrypt( &sCipher, dest, src, readlen );
      writelen = write( fdb, dest, readlen );
    }

  } while ( readlen > 0 );

  close( fda );
  close( fdb );    
}

int KCryptFile( char *filein, char *fileout ) {


  UCHAR pucRandom[200], pucCipherText[200], pucPlainText[200];
  RSAPublicKey Kpub;
  DWORD dwSize;
  LONG rv;
  int i, p, iReader, fd, result;
  int iList[16];
  char sUser[100];

  sprintf(sUser, "%s%s", getenv("HOME"), "/.muscle/public.key");

  /* Get the public key and generate the nonce */
  RSAKpubGetKey( &Kpub, sUser );  

  /* This will be the IDEA key */
  PMSCGenerateRandomNonce( pucRandom, 0x80 );

  /* Padding for correct crypto operation  */
  pucRandom[0] = 0x80; pucRandom[127] = 0x01;
	
  /* Encrypt the IDEA key with the public key */
  /* This will be prepended to the file    */
  dwSize = 0x80;
  memset( pucCipherText, 0x00, 0x80 );
  RSAKpubEncrypt( &Kpub, pucRandom, pucCipherText, &dwSize );

#ifdef PCSC_CRYPT_DEBUG
  printf("IDEA KEY: ");
  for (i=0; i < 0x80; i++) {
    printf("%x ", pucRandom[i]);
  } printf("\n");

  printf("IDEA CIPHER KEY: ");
  for (i=0; i < 0x80; i++) {
    printf("%x ", pucCipherText[i]);
  } printf("\n");
#endif

  CSCryptFile( filein, fileout, pucRandom, pucCipherText, 0x80 );

  return 0;
}
