/******************************************************************

        MUSCLE SmartCard Development ( http://www.linuxnet.com )
            Title  : csdecrypt.c
            Package: RSA Smartcard file encryption
            Author : David Corcoran
            Date   : 4/17/00
            Purpose: This decrypts system files using RSA private keys
	    and symmetric IDEA algorithm.

********************************************************************/

#ifdef __cplusplus
extern "C" {
#endif  

#include <includes.h>
#include <ssh.h>
#include <gmp.h>
#include <xmalloc.h>
#include <rsa.h>
#include <blowfish.h>
#include <stdio.h>
#include <sys/timeb.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef __cplusplus
}
#endif  

#include <winscard.h>
#include "kdecryptfile.h"

extern UCHAR ACCESS_CHV0_VAL[8];                                      
extern DWORD ACCESS_CHV0_SIZE;
extern UCHAR ACCESS_AUT0_KEY[8];  
extern DWORD ACCESS_AUT0_SIZE; 

void CSDeCryptFile( SCARDHANDLE hCard, char *srcfilename, 
		    char *destfilename ) {

  LONG rv;
  int fda, fdb, readlen, writelen, i;
  unsigned char src[200], dest[200];
  unsigned char Key[200];
  unsigned char CipherKey[200];
  struct stat fbuffer;
  CipherContext sCipher;
  RSAPublicKey Kpub;
  char cfile[200];
  char sUser[100];
  LONG filesize;
  LONG lByteCnt;
  DWORD dwSize;

  lByteCnt = 0;

  sprintf(sUser, "%s%s", getenv("HOME"), "/.muscle/public.key");

  /* Get the public key and generate the nonce */
  RSAKpubGetKey( &Kpub, sUser );  

  fdb = open( srcfilename, O_RDONLY );  
  fda = open( destfilename, O_CREAT | O_WRONLY, S_IRWXU | S_IROTH );

  if ( fdb <= 0 ) {
    printf("Error opening file: %s\n", srcfilename);
    return -1;
  }

  if ( fda <= 0 ) {
    printf("Error opening file: %s\n", destfilename);
    return -1;
  }

  readlen = read( fdb, src, 0x80 + 4 + 2 + 2 + 1 );

  /* Data Size */
  filesize = src[1]*0x01000000 + src[2]*0x00010000 + 
    src[3]*0x00000100 + src[4];

  memcpy( CipherKey, &src[9], src[5] );

#ifdef PCSC_CRYPT_DEBUG
  printf("IDEA CIPHER KEY : ");
  for ( i=0; i < src[5]; i++ ) {
    printf("%x ", CipherKey[i]);
  } printf("\n");
#endif

  /* Decrypt the synchronous key */
  dwSize = 0x80;
  memset( Key, 0x00, sizeof(Key) );
  rv = SCSRSASignData( hCard, CipherKey, Key, &dwSize );  

#ifdef PCSC_CRYPT_DEBUG
  printf("IDEA KEY : ");
  for ( i=0; i < src[5]; i++ ) {
    printf("%x ", Key[i]);
  } printf("\n");
#endif

  /* Set the key to it's context */
  cipher_set_key( &sCipher, SSH_CIPHER_IDEA, Key, 16, 0 );
  
  do {
    readlen = read( fdb, src, 16 ); 
  
    if ( readlen > 0 ) {
      /* Read, encrypt, and repeat */
      cipher_decrypt( &sCipher, dest, src, readlen );

      for (i=0; i < readlen; i++) {
	if ( lByteCnt > filesize ) {
	  return 1;
	}

	write( fda, &dest[i], 1 );
	lByteCnt += 1;
      }
    }
  } while ( readlen > 0 );

    /* Done */
    
  close(fda);
  close(fdb);
}

int KDecryptFile( SCARDHANDLE hCard, char *infile, char *outfile ) {

  LONG rv;

  /* Verify the transport key and the chv key
     Note - the chv will also be used in future releases 
  */
  rv = SCSVerifyKey( hCard, ACCESS_AUT0_SIZE, ACCESS_AUT0_KEY );
  rv = SCSVerifyChv( hCard, ACCESS_CHV0_SIZE, ACCESS_CHV0_VAL );

  /* Decrypt the file */
  CSDeCryptFile( hCard, infile, outfile );

  return 0;
}
