/*
 *
 * NAME
 *	pam_smartcard.c -- Copyright (C) 1999 David Corcoran
 *                      corcordt@cs.purdue.edu
 *
 * DESCRIPTION:
 *       This defines a PAM useable module for use with smartcards
 *
 * AUTHOR:
 *	David Corcoran <corcoran@linuxnet.com>, 4/13/00
 *
 * LICENSE: See file LICENSE.
 *
 */

#define PAM_SM_SESSION	

#include <security/pam_modules.h>
#include <includes.h>
#include <ssh.h>
#include <gmp.h>
#include <xmalloc.h>
#include <rsa.h>
#include <stdio.h>
#include <winscard.h>
#include <sys/timeb.h>
#include <unistd.h>

extern UCHAR ACCESS_CHV0_VAL[8];                                      
extern DWORD ACCESS_CHV0_SIZE;
extern UCHAR ACCESS_AUT0_KEY[8];  
extern DWORD ACCESS_AUT0_SIZE; 

void PMSCGenerateRandomNonce( PUCHAR pucBuffer, DWORD dwLength );


void PMSCGenerateRandomNonce( PUCHAR pucBuffer, DWORD dwLength ) {

  UCHAR ucRByte;
  int i;
  int iRandom;

  /* Start with a decent seed and then feed the random
     as the seed - use the Epoch and pid.
  */

  iRandom  = time(0) + getpid();

  for ( i=0; i < dwLength; i++ ) {
    srand( iRandom + i );
    ucRByte = (1 + (int)(255.0*rand()/(RAND_MAX+1.0)));
    pucBuffer[i] = ucRByte;
  }

  return;
}

PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int
				   argc, const char **argv) {

  LONG rv;
  SCARDCONTEXT hContext;
  SCARDHANDLE hCard;
  UCHAR pucRandom[200], pucCipherText[200], pucPlainText[200];
  RSAPublicKey Kpub;
  DWORD dwSize;
  DWORD dwProtocol, dwReaders;
  LPCSTR mszGroups;
  LPSTR mszReaders;
  SCARD_READERSTATE_A rgReaderStates;
  int i, p, iReader, fd, result;
  int iList[16];
  char KpubFile[100];
  char *sUser;
  char *sHost;

  pam_get_user( pamh, &sUser, "Login: " );
  sprintf( KpubFile, "%s%s%s", "/home/", sUser, "/.muscle/public.key" );

  /* Get the public key and generate the nonce */
  RSAKpubGetKey( &Kpub, KpubFile );  
  PMSCGenerateRandomNonce( pucRandom, 128 );

  /* Padding for correct crypto operation  */
  pucRandom[0] = 0x80; pucRandom[127] = 0x01;	

  /* Encrypt the nonce with the public key */
  dwSize = 128;
  memset( pucCipherText, 0x00, 128 );
  RSAKpubEncrypt( &Kpub, pucRandom, pucCipherText, &dwSize );

  /* Determine the source */
  pam_get_item( pamh, PAM_RHOST, (char *)&sHost );

  /* Connect to the smartcard subsystem */
  if ( sHost == 0 ) {
    SCardEstablishContext(SCARD_SCOPE_SYSTEM, 0, 0, &hContext);
  } else {
    SCardEstablishContext(SCARD_SCOPE_GLOBAL, (void *)sHost, 0, &hContext);
  }

  mszGroups = 0;
  SCardListReaders( hContext, mszGroups, 0, &dwReaders );
  mszReaders = (char *)malloc(sizeof(char)*dwReaders);
  SCardListReaders( hContext, mszGroups, mszReaders, &dwReaders );

  rgReaderStates.szReader       = mszReaders;
  rgReaderStates.dwCurrentState = SCARD_STATE_EMPTY;

  /* Wait for a card to be inserted */
  printf("Please insert your smartcard - ");
  fflush(stdout);
  SCardGetStatusChange( hContext, INFINITE, &rgReaderStates, 1 );

  printf("card inserted - authenticating user\n");

  SCardConnect( hContext, mszReaders, SCARD_SHARE_SHARED, 
		SCARD_PROTOCOL_T0, &hCard, &dwProtocol );

  /* Verify the transport key and the chv key
     Note - the chv will also be used in future releases 
  */
  rv = SCSVerifyKey( hCard, ACCESS_AUT0_SIZE, ACCESS_AUT0_KEY );
  rv = SCSVerifyChv( hCard, ACCESS_CHV0_SIZE, ACCESS_CHV0_VAL );

  /* Sign the crypted nonce using the card's private key */
  dwSize = 128;
  memset( pucPlainText, 0x00, sizeof(pucPlainText) );
  rv = SCSRSASignData( hCard, pucCipherText, pucPlainText, &dwSize );

  /* Disconnect from the smartcard subsystem */
  rv = SCardDisconnect( hCard, SCARD_LEAVE_CARD );
  rv = SCardReleaseContext( hContext );

  result = memcmp( pucRandom, pucPlainText, 128 );

  if ( result == 0 ) {
    return PAM_SUCCESS;
  } else {
    return PAM_AUTH_ERR;
  }

}

PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc,
			      const char **argv) {

return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc,
				const char **argv) {


return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
				   const char **argv) {

return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
				    const char **argv) {


return PAM_SUCCESS;
}

PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, 
				const char **argv) {

return PAM_SUCCESS;
}
